@@ -47,14 +47,17 @@ void HCApp::OnKeyUp(SDL_keysym key)
 		{
 		case SDLK_ESCAPE:
 			theApp->State(HCS_MENU);
+			theApp->ProcessStateChange();
 			break;
 
 		case SDLK_p:
 			theApp->State(HCS_PAUSED);
+			theApp->ProcessStateChange();
 			break;
 
 		case SDLK_F1:
 			theApp->State(HCS_PAUSED);
+			theApp->ProcessStateChange();
 			theApp->InitHelp();
 			break;
 
@@ -63,17 +66,25 @@ void HCApp::OnKeyUp(SDL_keysym key)
 		} // switch (key)
 		break;
 
+	case HCS_ENDLEVEL:
+		break;
+
 	case HCS_GAMEOVER:
 		switch (key.sym)
 		{
 			// Skip game over screen
 		case SDLK_RETURN:
 			theApp->State(HCS_PLAYING);
+			theApp->ProcessStateChange();
 			break;
 
 			// Goes to main menu
 		case SDLK_ESCAPE:
 			theApp->State(HCS_MENU);
+			theApp->ProcessStateChange();
+			break;
+
+    default:
 			break;
 		}
 		break;
@@ -84,6 +95,7 @@ void HCApp::OnKeyUp(SDL_keysym key)
 			// Skip script
 		case SDLK_RETURN:
 			theApp->State(HCS_ENDLEVEL);
+			theApp->ProcessStateChange();
 			break;
 
 		case SDLK_SPACE:
@@ -95,10 +107,15 @@ void HCApp::OnKeyUp(SDL_keysym key)
 
 		case SDLK_ESCAPE:
 			theApp->State(HCS_MENU);
+			theApp->ProcessStateChange();
 			break;
 
 		case SDLK_p:
 			theApp->State(HCS_PAUSED);
+			theApp->ProcessStateChange();
+			break;
+
+    default:
 			break;
 		}
 		break;
@@ -115,6 +132,7 @@ void HCApp::OnKeyUp(SDL_keysym key)
 			}
 
 			theApp->State(theApp->lastState);
+			theApp->ProcessStateChange();
 			break;
 
 		default:
@@ -132,6 +150,9 @@ void HCApp::OnKeyUp(SDL_keysym key)
 			case SDLK_p:
 				theApp->DestroyHelp();
 				break;
+
+  		default:
+	  		break;
 			}
 		}
 		else
@@ -140,9 +161,11 @@ void HCApp::OnKeyUp(SDL_keysym key)
 		}
 		break;
 
+	default:
 	case HCS_INTRO:
 	case HCS_CREDITS:
 		theApp->State(HCS_MENU);
+		theApp->ProcessStateChange();
 		break;
 	} // switch (appState)
 }
@@ -160,11 +183,12 @@ HCApp::HCApp() : JApp("Holotz's Castle", 640, 480, false, 16, SDL_HWSURFACE | SD
 	textIntro = 0;
 	menu = 0;
 	imgMenu = 0;
-	state = HCS_INTRO;
+	state = lastState = HCS_INTRO;
 	memset(imgCredits, 0, sizeof(imgCredits));
 	memset(textCredits, 0, sizeof(textCredits));
 	fps = 25;
 	playlistName = 0;
+	stateChanged = false;
 }
 
 bool HCApp::Init(s32 argc, char **argv)
@@ -226,7 +250,8 @@ bool HCApp::Init(s32 argc, char **argv)
 	MouseCursor(false);
 
 	State(HCS_INTRO);
-
+	ProcessStateChange();
+ 
 	return true;
 }
 
@@ -333,7 +358,7 @@ bool HCApp::InitMenu()
 		
 		for (s32 sn = 1; sn < playlist.Size(); ++sn)
 		{
-			it->AddNodeGo(new JTextMenuEntry(playlist[sn], &OnNew, (void *)sn));                                 // Story N
+			it->AddNodeGo(new JTextMenuEntry(playlist[sn], &OnNew, JCAST_S32_TO_VOIDPTR(sn)));                   // Story N
 		}
 
 		it->Parent();
@@ -355,7 +380,7 @@ bool HCApp::InitMenu()
 					for (s32 i = 1; i < preferences.NumVideoModes(); ++i)
 					{
 						snprintf(str, 128, "%dx%d", preferences.VideoModes()[i].w, preferences.VideoModes()[i].h);
-						it->AddNodeGo(new JTextMenuEntry(str, &OnVideoMode, (void *)i));                               //    Modes
+						it->AddNodeGo(new JTextMenuEntry(str, &OnVideoMode, JCAST_S32_TO_VOIDPTR(i)));                 //    Modes
 					}
 
 					it->Parent();
@@ -416,7 +441,7 @@ bool HCApp::InitMenu()
 				it->AddBranchGo(new JTextMenuEntry(preferences.Langs()[0], &OnLanguage, (void *)0));               //   Available langs
 				for (s32 c = 1; c < preferences.NumLangs(); ++c)
 				{                                                
-					it->AddNodeGo(new JTextMenuEntry(preferences.Langs()[c], &OnLanguage, (void *)c));               //   Available langs
+					it->AddNodeGo(new JTextMenuEntry(preferences.Langs()[c], &OnLanguage, JCAST_S32_TO_VOIDPTR(c))); //   Available langs
 				}
 
 				it->Parent();
@@ -449,7 +474,7 @@ bool HCApp::InitMenu()
 			else
 			{ 
 				snprintf(strLev, sizeof(strLev), "< %s - %d >", saveData[i].Story(), saveData[i].Level());
-				it->AddNodeGo(new JTextMenuEntry(strLev, &OnLoad, (void *)i));                                     //   Slot N
+				it->AddNodeGo(new JTextMenuEntry(strLev, &OnLoad, JCAST_S32_TO_VOIDPTR(i)));                       //   Slot N
 			}
 		}
 		it->Parent();
@@ -473,12 +498,12 @@ bool HCApp::InitMenu()
 			if (strlen(saveData[i].Story()) == 0)
 			{
 				// Empty slot
-				it->AddNodeGo(new JTextMenuEntry("< ******* >", &OnSave, (void *)i));                              //    ...
+				it->AddNodeGo(new JTextMenuEntry("< ******* >", &OnSave, JCAST_S32_TO_VOIDPTR(i)));                //    ...
 			}
 			else
 			{ 
 				snprintf(strLev, sizeof(strLev), "< %s - %d >", saveData[i].Story(), saveData[i].Level());
-				it->AddNodeGo(new JTextMenuEntry(strLev, &OnSave, (void *)i));                                     //   Slot N
+				it->AddNodeGo(new JTextMenuEntry(strLev, &OnSave, JCAST_S32_TO_VOIDPTR(i)));                       //   Slot N
 			}
 		}
 		it->Parent();
@@ -594,10 +619,25 @@ bool HCApp::InitPaused()
 
 	imgBack->Alpha(255);
 	JDELETE(textBack);
+
+	// Reads the pause file in the correct language
+	char str[256];
+	snprintf(str, sizeof(str), HC_DATA_DIR "messages/%s/messages.txt", preferences.LangCodes()[preferences.CurLang()]);
+
+	JTextFile f;
+	if (!f.Load(str, "rb"))
+	{
+		snprintf(str, sizeof(str), "... ZzZzZz ...");
+	}
+	else
+	{
+		f.NextLine();
+		f.ReadLine(str);
+	}
+
 	textBack = new HCText;
-	textBack->Init(HCTEXTTYPE_NARRATIVE, "... ZzZzZz ...", &level.Theme(), &fontLarge, JFONTALIGN_CENTER, false, 0);
-	textBack->Pos((Width() - textBack->Image().Width())/2,
-								(Height() - textBack->Image().Height())/2);
+	textBack->Init(HCTEXTTYPE_NARRATIVE, str, &level.Theme(), &fontLarge, JFONTALIGN_CENTER, false, 0);
+	textBack->Pos((Width() - textBack->Image().Width())/2, (Height() - textBack->Image().Height())/2);
 	
 	return true;
 }
@@ -620,13 +660,14 @@ bool HCApp::InitPlaying()
 			// Game end
 		case -1:
 			State(HCS_CREDITS);
+			ProcessStateChange();
 			return true;
 	}
 
 	// Checks for an existing script
 	JString str;
 	str.Format("%s%s/script/%s/level%03d.hcs", playlist.StoryDir(), playlist.StoryName(), preferences.LangCodes()[preferences.CurLang()], levelNumber);
-
+	
 	if (JFile::Exists(str))
 	{
 		if (!script.Load(str))
@@ -638,6 +679,7 @@ bool HCApp::InitPlaying()
 		{
 			level.Scripted(true);
 			State(HCS_SCRIPT);
+			ProcessStateChange();
 		}
 	}
 	else
@@ -706,7 +748,6 @@ bool HCApp::InitCredits()
 																							 "Holotz's Castle\n(C) Juan Carlos Seijo Pérez 2004",
 																							 ACKNOWLEDGEMENTS};
 
-	s32 yOff = height;
 	bool ok;
 	ok = true;
 	for (s32 i = 0; ok && i < HCCREDITS_IMAGE_COUNT; ++i)
@@ -884,11 +925,11 @@ bool HCApp::InitHelp()
 	if (!imgHelp)
 	{
 		fprintf(stderr, "Couldn't create help image, out of memory. Check manual.\n");
-		JDELETE(strText);
+		JDELETE_ARRAY(strText);
 		return false;
 	}
 
-	JDELETE(strText);
+	JDELETE_ARRAY(strText);
 	imgHelp->Pos((Width() - imgHelp->Width())/2,
 							 (Height() - imgHelp->Height())/2);
 	imgHelp->Alpha(128);
@@ -960,7 +1001,7 @@ void HCApp::OnSound(void *data)
 
 void HCApp::OnNew(void *data)
 {
-	if (theApp->playlist.GoTo(theApp->playlist[(long)data]))
+	if (theApp->playlist.GoTo(theApp->playlist[JCAST_VOIDPTR_TO_S32(data)]))
 	{
 		theApp->levelNumber = 1;
 		theApp->State(HCS_PLAYING);
@@ -981,10 +1022,10 @@ void HCApp::OnHelp(void *data)
 void HCApp::OnVideoMode(void *data)
 {
 	HCApp *a = (HCApp *)App();
-	a->preferences.VideoMode((long)data);
+	a->preferences.VideoMode(JCAST_VOIDPTR_TO_S32(data));
 	a->preferences.Save();
-	App()->Resize(a->preferences.VideoModes()[(long)data].w, 
-								a->preferences.VideoModes()[(long)data].h, 
+	App()->Resize(a->preferences.VideoModes()[JCAST_VOIDPTR_TO_S32(data)].w, 
+								a->preferences.VideoModes()[JCAST_VOIDPTR_TO_S32(data)].h, 
 								a->preferences.Fullscreen());
 	a->State(HCS_MENU);
 }
@@ -992,9 +1033,9 @@ void HCApp::OnVideoMode(void *data)
 void HCApp::OnBPP(void *data)
 {
 	HCApp *a = (HCApp *)App();
-	a->preferences.BPP((long)data);
+	a->preferences.BPP(JCAST_VOIDPTR_TO_S32(data));
 	a->preferences.Save();
-	a->depth = (long)data;
+	a->depth = JCAST_VOIDPTR_TO_S32(data);
 	App()->Resize(a->preferences.VideoModes()[a->preferences.VideoMode()].w, 
 								a->preferences.VideoModes()[a->preferences.VideoMode()].h,
 								a->preferences.Fullscreen());
@@ -1004,7 +1045,7 @@ void HCApp::OnBPP(void *data)
 void HCApp::OnWindowMode(void *data)
 {
 	HCApp *a = (HCApp *)App();
-	a->preferences.Fullscreen((long)data);
+	a->preferences.Fullscreen(JCAST_VOIDPTR_TO_S32(data));
 	a->preferences.Save();
 	App()->Resize(a->preferences.VideoModes()[a->preferences.VideoMode()].w, 
 								a->preferences.VideoModes()[a->preferences.VideoMode()].h,
@@ -1015,7 +1056,7 @@ void HCApp::OnWindowMode(void *data)
 void HCApp::OnLanguage(void *data)
 {
 	HCApp *a = (HCApp *)App();
-	a->preferences.CurLang((long)data);
+	a->preferences.CurLang(JCAST_VOIDPTR_TO_S32(data));
 	a->preferences.Save();
 	a->State(HCS_MENU);
 }
@@ -1025,8 +1066,8 @@ void HCApp::OnDefaults(void *data)
 	HCApp *a = (HCApp *)App();
 	a->preferences.Reset();
 	a->preferences.Save();
-	App()->Resize(a->preferences.VideoModes()[(long)data].w, 
-								a->preferences.VideoModes()[(long)data].h, 
+	App()->Resize(a->preferences.VideoModes()[JCAST_VOIDPTR_TO_S32(data)].w, 
+								a->preferences.VideoModes()[JCAST_VOIDPTR_TO_S32(data)].h, 
 								a->preferences.Fullscreen());
 	a->State(HCS_MENU);
 }
@@ -1034,9 +1075,9 @@ void HCApp::OnDefaults(void *data)
 void HCApp::OnLoad(void *data)
 {
 	HCApp *app = (HCApp *)App();
-	if (app->playlist.GoTo(app->saveData[(long)data].Story()))
+	if (app->playlist.GoTo(app->saveData[JCAST_VOIDPTR_TO_S32(data)].Story()))
 	{
-		app->levelNumber = app->saveData[(long)data].Level();
+		app->levelNumber = app->saveData[JCAST_VOIDPTR_TO_S32(data)].Level();
 		app->State(HCS_PLAYING);
 	}
 }
@@ -1044,7 +1085,7 @@ void HCApp::OnLoad(void *data)
 void HCApp::OnSave(void *data)
 {
 	HCApp *app = (HCApp *)App();
-	if (app->saveData[(long)data].Save((long)data, app->playlist.StoryName(), app->levelNumber))
+	if (app->saveData[JCAST_VOIDPTR_TO_S32(data)].Save(JCAST_VOIDPTR_TO_S32(data), app->playlist.StoryName(), app->levelNumber))
 	{
 		app->State(HCS_MENU);
 	}
@@ -1274,11 +1315,13 @@ bool HCApp::UpdatePlaying()
 		// Player died
 		case 1:
 			State(HCS_GAMEOVER);
+			ProcessStateChange();
 			break;
 		
 		// Level ended
 		case 2:
 			State(HCS_ENDLEVEL);
+			ProcessStateChange();
 			break;
 		
 		// Nothing happened
@@ -1323,6 +1366,7 @@ bool HCApp::UpdateIntro()
 		{
 			// Done, show menu
 			State(HCS_MENU);
+			ProcessStateChange();
 
 			return true;
 		}
@@ -1409,6 +1453,7 @@ bool HCApp::UpdateCredits()
 		{
 			// Done with the credits
 			State(HCS_MENU);
+			ProcessStateChange();
 		}
 	}
 	
@@ -1424,6 +1469,7 @@ bool HCApp::UpdateScript()
 	if (script.Finished())
 	{
 		State(HCS_ENDLEVEL);
+		ProcessStateChange();
 	}
 
   return true;
@@ -1441,6 +1487,7 @@ bool HCApp::UpdateEndLevel()
 		// Next level
 		++levelNumber;
 		State(HCS_PLAYING);
+		ProcessStateChange();
 	}
 	
 	return true;
@@ -1459,10 +1506,23 @@ bool HCApp::UpdateGameOver()
 			imgBack->Paste(&img, 0, 0, img.Width(), img.Height());
 			imgBack->Alpha(255);
 			
-			
+			// Reads the pause file in the correct language
+			char str[256];
+			snprintf(str, sizeof(str), HC_DATA_DIR "messages/%s/messages.txt", preferences.LangCodes()[preferences.CurLang()]);
+
+			JTextFile f;
+			if (!f.Load(str, "rb"))
+			{
+				snprintf(str, sizeof(str), "Game Over");
+			}
+			else
+			{
+				f.ReadLine(str);
+			}
+
 			textBack = new HCText;
 			
-			if (!textBack->Init(HCTEXTTYPE_NARRATIVE, "GaMe oVeR", &level.Theme(), &fontLarge, JFONTALIGN_CENTER, false, 0))
+			if (!textBack->Init(HCTEXTTYPE_NARRATIVE, str, &level.Theme(), &fontLarge, JFONTALIGN_CENTER, false, 0))
 			{
 				return false;
 			}
@@ -1490,6 +1550,7 @@ bool HCApp::UpdateGameOver()
 				else
 				{
 					State(HCS_PLAYING);
+					ProcessStateChange();
 				}
 			}
 		}
@@ -1504,38 +1565,54 @@ bool HCApp::UpdateGameOver()
 
 bool HCApp::Update()
 {
+	bool ret = false;
+
 	UpdateEvents();
 
-
+	if (stateChanged)
+	{
+		ProcessStateChange();
+	}
+	
 	switch (state)
 	{
 		case HCS_MENU:
-			return UpdateMenu();
+			ret = UpdateMenu();
+      break;
 
 		case HCS_PLAYING:
-			return UpdatePlaying();
+			ret = UpdatePlaying();
+      break;
 
 		case HCS_PAUSED:
-			return UpdatePaused();
+			ret = UpdatePaused();
+      break;
 
 		case HCS_INTRO:
-			return UpdateIntro();
+			ret = UpdateIntro();
+      break;
 
 		case HCS_CREDITS:
-			return UpdateCredits();
+			ret = UpdateCredits();
+      break;
 
 		case HCS_SCRIPT:
-			return UpdateScript();
+			ret = UpdateScript();
+      break;
 
 		case HCS_GAMEOVER:
-			return UpdateGameOver();
+			ret = UpdateGameOver();
+      break;
 
 		case HCS_ENDLEVEL:
-			return UpdateEndLevel();
+			ret = UpdateEndLevel();
+      break;
 
 		default:
 			return false;
 	}
+
+	return ret;
 }
 
 s32 HCApp::LoadWorld()
@@ -1561,9 +1638,9 @@ s32 HCApp::LoadWorld()
 		}
 	}
 
-	JFile f;
-	if (!f.Open(str, "rb") || 
-			0 != level.Load(f))
+	JRW f;
+	if (!f.Create(str, "rb") || 
+			0 != level.Load(f, str))
 	{
 		fprintf(stderr, "Error loading the level %s.\n", str.Str());
 		return 0;
@@ -1584,6 +1661,12 @@ void HCApp::State(HCState newState)
 {
 	lastState = state;
 	state = newState;
+  stateChanged = true;
+}
+
+void HCApp::ProcessStateChange()
+{
+	stateChanged = false;
 
 	switch (lastState)
 	{
@@ -1635,6 +1718,7 @@ void HCApp::State(HCState newState)
 			{
 				fprintf(stderr, "Failed to init intro. Check the manual.\n");
 				State(HCS_MENU);
+				ProcessStateChange();
 			}
 			break;
 
@@ -1662,6 +1746,7 @@ void HCApp::State(HCState newState)
 			{
 				fprintf(stderr, "Failed to init pause. Check the manual.\n");
 				State(HCS_PLAYING);
+				ProcessStateChange();
 			}
 			break;
 			
@@ -1670,6 +1755,7 @@ void HCApp::State(HCState newState)
 			{
 				fprintf(stderr, "Failed to init game over. Check the manual.\n");
 				State(HCS_PLAYING);
+				ProcessStateChange();
 			}
 			break;
 
@@ -1678,6 +1764,7 @@ void HCApp::State(HCState newState)
 			{
 				fprintf(stderr, "Failed to init end of level. Check the manual.\n");
 				State(HCS_PLAYING);
+				ProcessStateChange();
 			}
 			break;
 
@@ -1686,6 +1773,7 @@ void HCApp::State(HCState newState)
 			{
 				fprintf(stderr, "Failed to init credits. Check the manual.\n");
 				State(HCS_MENU);
+				ProcessStateChange();
 			}
 			break;
 
@@ -1724,6 +1812,8 @@ void HCApp::Destroy()
 	{
 		JDELETE(textCredits[i]);
 	}
+	
+	HCUtil::Destroy();
 }
 
 // Main function
@@ -1742,7 +1832,7 @@ int main(int argc, char **argv)
 
 void HCApp::PrintUsage(char *program)
 {
-	fprintf(stderr, "Holotz's Castle v1.3.1 (C) Juan Carlos Seijo Pérez - 2004.\n\n");
+	fprintf(stderr, "Holotz's Castle v1.3.7 (C) Juan Carlos Seijo Pérez - 2004.\n\n");
 	fprintf(stderr, "Usage: %s [-p playlist]", program);
 	fprintf(stderr, " [-f]ullscreen [-w]indowed [--fps nnn] [-mWxHxBPP]\n");
 	fprintf(stderr, "\n");
