@@ -70,7 +70,9 @@ class HCEnemyChaser;
 "Andreas Jochens (AMD64 Patches),\n"\
 "Maksim Mokriyev (Ukranian translation)\n"\
 "Filip (Triple T) Bártek (Czech translation)\n"\
-"Pierre-Paul Lavoie (FreeBSD porting)"
+"Pierre-Paul Lavoie (FreeBSD porting)\n"\
+"Filipe Silva (Portuguese translation)\n"\
+"Ben Wong (English translation, patches)"
 
 /** Possible states of the game.
  */
